package com.loafle.commons.rpc.protocol.grpc;

import java.lang.reflect.Type;
import java.util.List;

import com.google.gson.Gson;
import com.google.protobuf.ByteString;
import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.protocol.RPCRegistryCodec;

/**
 * JSONRPCServerRequestCodec
 */
class GRPCServerRequestCodec implements RPCRegistryCodec {
  private Gson gson;
  private String serviceName;
  private String methodName;
  private List<ByteString> params;


  public GRPCServerRequestCodec(Gson gson, String service, String method, List<ByteString> params) throws RPCException {
    this.gson = gson;
    this.serviceName = service;
    this.methodName = method;
    this.params = params;
  }

  public String method() {
    return String.format("%s.%s", this.serviceName, this.methodName);
  }

  public Object[] params(Type[] paramTypes) throws RPCException {
    if (null == paramTypes && null == this.params) {
      return null;
    }

    if ((null != paramTypes && null == this.params) || null == paramTypes && null != this.params) {
      throw new RPCException("params is not valied");
    }

    if (paramTypes.length != this.params.size()) {
      throw new RPCException(String.format("The size of params[%d] and types[%d] is not same", this.params.size(), paramTypes.length));
    }

    Object[] result = new Object[paramTypes.length];

    for (int i = 0; i < paramTypes.length; i++) {
      Type paramType = paramTypes[i];
      String param = this.params.get(i).toStringUtf8();

      if (paramType.equals(String.class)) {
        result[i] = param;
        continue;
      }

      result[i] = this.gson.fromJson(param, paramType);
    }

    return result;
  }
}