package com.loafle.commons.rpc.protocol.json;

import com.google.gson.Gson;
import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.protocol.RPCServerCodec;
import com.loafle.commons.rpc.protocol.RPCServerRequestCodec;

/**
 * JSONRPCServerCodec
 */
public class JSONRPCServerCodec implements RPCServerCodec {
  private Gson gson;

  public JSONRPCServerCodec() {
    this.gson = new Gson();
  }
  public JSONRPCServerCodec(Gson gson) {
    this.gson = gson;
  }

  public void setGson(Gson gson) {
    this.gson = gson;
  }

  public Gson getGson() {
    return this.gson;
  }

  public RPCServerRequestCodec request(byte[] buff) throws RPCException {
    return new JSONRPCServerRequestCodec(this.gson, buff);
  }

  public byte[] notification(String method, Object... params) throws RPCException {
    JSONServerNotification notification = new JSONServerNotification(method, params);
    JSONServerResponse response = new JSONServerResponse(JSONRPC.version, notification, null);

    String json = this.gson.toJson(response);
    return json.getBytes();
  }
}
