package com.loafle.commons.rpc.protocol.json;

import java.lang.reflect.Type;

import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;
import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.protocol.RPCError;
import com.loafle.commons.rpc.protocol.RPCServerRequestCodec;

/**
 * JSONRPCServerRequestCodec
 */
class JSONRPCServerRequestCodec implements RPCServerRequestCodec {
  private Gson gson;
  private JSONServerRequest request;

  public JSONRPCServerRequestCodec(Gson gson, byte[] buff) throws RPCException {
    this.gson = gson;
    String json = new String(buff);
    try {
      this.request = this.gson.fromJson(json, JSONServerRequest.class);
    } catch (JsonSyntaxException e) {
      throw new RPCException("request is not valied", json);
    }
  }

  public String method() {
    return this.request.method;
  }

  public Object[] params(Type[] paramTypes) throws RPCException {
    if (null == paramTypes && null == this.request.params) {
      return null;
    }

    if ((null != paramTypes && null == this.request.params) || null == paramTypes && null != this.request.params) {
      throw new RPCException("params is not valied");
    }

    if (paramTypes.length != this.request.params.size()) {
      throw new RPCException(String.format("The size of params[%d] and types[%d] is not same", this.request.params.size(), paramTypes.length));
    }

    Object[] result = new Object[paramTypes.length];

    for (int i = 0; i < paramTypes.length; i++) {
      Type paramType = paramTypes[i];
      String param = this.request.params.get(i);

      result[i] = this.gson.fromJson(param, paramType);
    }

    return result;
  }

  public boolean hasResponse() {
    return null != this.request.id;
  }

  public byte[] response(Object reply, RPCException ex) throws RPCException {
    RPCError rpcError = null;
    if (null != ex) {
      rpcError = new RPCError(1, ex.getMessage(), ex.getData());
    }
    JSONServerResponse response = new JSONServerResponse(JSONRPC.version, reply, rpcError);
    response.id = this.request.id;

    String json = this.gson.toJson(response);
    return json.getBytes();
  }
}